using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace AsusTrxExtract
{
    public partial class Form1 : Form
    {
        #region Members

        private ImageFile imageFile = null;

        #endregion

        #region Constructors

        public Form1()
        {
            InitializeComponent();
        }

        #endregion

        #region Properties

        public bool BrowsingEnabled
        {
            get
            {
                return this.btnFile.Enabled;
            }
            set
            {
                this.txtFile.Enabled = value;
                this.btnFile.Enabled = value;
                if (this.txtFile.Text.Trim().Length == 0)
                {
                    this.btnLoad.Enabled = false;
                }
                else
                {
                    this.btnLoad.Enabled = value;
                }
                this.btnExtract.Enabled = !value;
                this.lnkBuildInfo.Enabled = !value;
            }
        }

        #endregion

        #region EventHandler

        private void btnFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Title = "Browse for image file";
            ofd.Filter = "Asus image file (*.nas)|*.nas|All files (*.*)|*.*";
            ofd.DefaultExt = "nas";
            if (ofd.ShowDialog(this) == DialogResult.OK)
            {
                if (File.Exists(ofd.FileName))
                {
                    this.txtFile.Text = ofd.FileName;
                }
                else
                {
                    MessageBox.Show(this, String.Format("File {0} does not exist!", ofd.FileName), "File error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            ofd.Dispose();
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            loadImageFile(this.txtFile.Text.Trim());
            if (imageFile != null)
            {
                fillImageInfo();
            }
        }

        private void btnExtract_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            try
            {
                imageFile.writeImageFilesToDisk(this.txtFile.Text);
                MessageBox.Show(this, "Image files successfully extracted.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, String.Format("Error extracting images: {0}", ex.Message), "File error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            imageFile = null;
            clearImageInfo();
            this.BrowsingEnabled = true;
        }

        private void txtFile_TextChanged(object sender, EventArgs e)
        {
            if (this.txtFile.Text.Length == 0)
            {
                this.btnLoad.Enabled = false;
            }
            else
            {
                this.btnLoad.Enabled = true;
            }
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Clipboard.Clear();
            Clipboard.SetText(imageFile.BuildInfo);
        }

        #endregion

        #region Methods

        void loadImageFile(String fileName)
        {
            Cursor.Current = Cursors.WaitCursor;

            FileStream fs = null;
            try
            {
                fs = File.OpenRead(fileName);
                this.txtFile.Text = fileName;
                byte[] data = new byte[fs.Length];
                fs.Read(data, 0, (int) fs.Length);
                fs.Close();
                fs.Dispose();

                try
                {
                    imageFile = new ImageFile(data);
                    this.BrowsingEnabled = false;
                }
                catch (ArgumentException ex)
                {
                    MessageBox.Show(this, String.Format("File {0} is not a valid image file ({1})!", fileName, ex.Message), "Image error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    imageFile = null;
                    return;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(this, String.Format("Error reading image file: {0}", ex.Message), "Image error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    imageFile = null;
                    return;
                }
                finally
                {
                    Cursor.Current = Cursors.Default;
                }
            }
            catch (FileNotFoundException)
            {
                MessageBox.Show(this, String.Format("File {0} does not exist!", fileName), "File error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (IOException ex)
            {
                MessageBox.Show(this, String.Format("Error opening file {0}: {1}!", fileName, ex.Message), "File error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        void fillImageInfo()
        {
            this.txtName.Text = imageFile.Name;
            this.txtVersion.Text = imageFile.Version;
            this.txtBuildDate.Text = imageFile.BuildDate;
            this.txtBuildBy.Text = imageFile.BuildBy;
            this.txtFlashSize.Text = imageFile.FlashImageSize.ToString();
            this.txtDiskSize.Text = imageFile.DiskImageSize.ToString();

            toolTip.SetToolTip(this.lnkBuildInfo, string.Format("(click to copy to clipboard)\n\n{0}", imageFile.BuildInfo));
        }

        void clearImageInfo()
        {
            this.txtName.Clear();
            this.txtVersion.Clear();
            this.txtBuildDate.Clear();
            this.txtBuildBy.Clear();
            this.txtFlashSize.Clear();
            this.txtDiskSize.Clear();
        }
        
        #endregion
    }
}